<?php

namespace Devsense\PHP\Editor {

    /**
     * Provides access to IDE information.
     */
    interface IIdeController {

        /**
         * Outputs given text to IDE's output console. Used for debugging.
         * @param string $text 
         */
        function output($text);

        // TODO: information about project, file, additional information about class, function, select AST nodes
    }

    /**
     * Describes code block within text editor.
     */
    class EditorBlock {
        /**
         * Language name. PHP, Text, etc.
         * @var string
         */
        var $Language;

        /**
         * Start position.
         * @var int
         */
        var $Start;

        /**
         * Length of block.
         * @var int
         */
        var $Length;

        /**
         * Additional content to be virtually prepended to block content.
         * @var string|null
         */
        var $Prefix;

        /**
         * Additional content to be virtually appended to block content.
         * @var string|null
         */
        var $Suffix;

        /**
         * Initializes editor language block.
         * @param string|object $language 
         * @param int $start 
         * @param int $length 
         * @param string|null $prefix 
         * @param string|null $suffix 
         */
        function __construct($language, $start, $length, $prefix, $suffix) {
            $this->Language = is_string($language) ? $language : ("\\" . get_class($language));
            $this->Start = $start;
            $this->Length = $length;
            $this->Prefix = $prefix;
            $this->Suffix = $suffix;
        }
    }

    /**
     * Specifies classification of a word. Case insensitive.
     */
    class EditorWordColor
    {
        /**
         * Word to be colored. Empty string for default classification of anything unspecified.
         * @var string
         */
        var $Word;

        /**
         * Word classification, e.g. keyword, comment, string.
         * @var mixed
         */
        var $Type;

        function __construct($word, $type)
        {
        	$this->Word = $word;
            $this->Type = $type;
        }
        
    }

    /**
     * Represents node information to be handled by Code Sense.
     */
    class NodeInfo {
        /**
         * Name of the element.
         * @var string
         */
        var $ElementName;

        /**
         * Expression code.
         * @var string
         */
        var $Text;

        /**
         * How is the node used.
         * @var string
         */
        var $Usage;

        /**
         * In case of a type member, gets array of its possible names.
         * @var string[]
         */
        var $MemberOf;

        /**
         * Current function name.
         * @var string
         */
        var $Routine;

        function __construct($o) {
            $this->ElementName = $o->ElementName;
            $this->Text = $o->Text;
            $this->Usage = $o->Usage;
            $this->MemberOf = $o->MemberOf;
            $this->Routine = $o->Routine;
        }
    }

    /**
     * Provides editor with additional mapping of language blocks.
     */
    interface IEditorBlocks {
        /**
         * Gets additional mapping of language blocks.
         * @param string $source Source code text.
         * @return EditorBlock[]
         */
        function GetBlocks($source);

        /**
         * Gets list of keywords that when appear or disappear in code, updating of language blocks is triggered.
         * @return string[] List of keywords that opens and closes language block. (e.g. ['{{', '}}', '{#', '#}'])
         */
        function GetTriggers();
    }

    /**
     * Provides editor with syntax colorization.
     */
    interface IEditorClassifier {
        ///**
        // * Gets tokens within given source.
        // * @param string $source Source code text.
        // * @return EditorClassifySpan[]
        // */
        //function GetSpans($source);

        /**
         * Gets colors (classifications) of specific words to be automatically colored in source.
         * @return EditorWordColor[] Classification words.
         */
        function GetColors();
    }

    /**
     * Extends editor Code Sense functionality.
     */
    interface IPhpEditorCodeSense {

        /**
         * Gets additional tooltip text for given code element.
         * @param NodeInfo $node Code element information.
         */
        function GetToolTip(NodeInfo $node);

        /**
         * Gets URL pointing to help for given node.
         * @param NodeInfo $node 
         */
        function GetHelpUrl(NodeInfo $node);
    }

    /**
     * Provides editor with additional warnings, errors and notices in user's code.
     */
    interface IPhpEditorValidator { }

    /**
     * Provides editor with additional code snippets.
     */
    interface IPhpEditorSnippets { }
}