<?php

class StackTraceFilter
{
    /**
     * Filters stack frames from PHPUnit classes.
     *
     * @param  Exception $e
     * @param  boolean   $asString
     * @return string
     */
    public static function getFilteredStacktrace(Exception $e, $asString = true)
    {
        $prefix = false;
        $script = realpath($GLOBALS['_SERVER']['SCRIPT_NAME']);

        if (defined('__PHPUNIT_PHAR__')) {
            $prefix = 'phar://' . __PHPUNIT_PHAR__ . '/';
        }

        if ($asString === true) {
            $filteredStacktrace = '';
        } else {
            $filteredStacktrace = array();
        }

        if ($e instanceof PHPUnit_Framework_SyntheticError) {
            $eTrace = $e->getSyntheticTrace();
            $file  = $e->getSyntheticFile();
            $line  = $e->getSyntheticLine();
        } else {
            if ($e->getPrevious()) {
                $eTrace = $e->getPrevious()->getTrace();
            } else {
                $eTrace = $e->getTrace();
            }
            $file  = $e->getFile();
            $line  = $e->getLine();
        }

        foreach ($eTrace as $frame) {
            // record the trace if it contains function information
            if (isset($file) && $file !== $script && is_file($file) &&
                !self::isFileBlacklisted($file) && //isset($blacklist[$file]) &&
                strpos($file, $prefix) !== 0 &&
                strpos($file, 'phar://') === false &&
                isset($frame['function'])) {
                if ($asString === true) {
                    /*
                     *
                     * at (?<methodName>[^\\\\]+) in (?<filename>.+):line (?<linenumber>\\d+)
                     *
                     */

                    $name = (isset($frame['class']) ? ($frame['class'] . '::') : '') . $frame['function']; // . '(' . $frame['args'] . ')';
                    $name = strtr($name, '\\', '/');
                    $filteredStacktrace .= sprintf(
                        "at %s in %s:line %d\n",
                        $name,
                        $file,
                        $line ?: 0
                    );

                } else {
                    $filteredStacktrace[] = $frame;
                }
            }

            // file:line corresponds to the next frame
            $file = isset($frame['file']) ? $frame['file'] : null;
            $line = isset($frame['line']) ? $frame['line'] : null;
        }

        return $filteredStacktrace;
    }

    /**
     * @param  string  $file
     * @return boolean
     */
    private static function isFileBlacklisted($file)
    {
        if (!defined('PHPUNIT_TESTSUITE')) {
            static $blacklist;
            if (class_exists('PHPUnit_Util_Blacklist')) {
                // phpunit v.4.0+
                if (!isset($blacklist)) {
                    $blacklist = new PHPUnit_Util_Blacklist();
                }
                return $blacklist->isBlacklisted($file);
            } else {
                // phpunit v.3.7
                if (!isset($blacklist)) {
                    $blacklist = PHPUnit_Util_GlobalState::phpunitFiles();
                }
                return isset($blacklist[$file]);
            }
        }
        return false;
    }

    /**
     * @param  array  $trace
     * @param  string $file
     * @param  int    $line
     * @return boolean
     * @since  Method available since Release 3.3.2
     */
    public static function frameExists(array $trace, $file, $line)
    {
        foreach ($trace as $frame) {
            if (isset($frame['file']) && $frame['file'] == $file &&
                isset($frame['line']) && $frame['line'] == $line) {
                return true;
            }
        }

        return false;
    }
}

/**
 * A TestListener that generates output suitable for the VS.NET test adapter.
 */
class PhpTestingPrinter extends PHPUnit_Util_Printer implements PHPUnit_Framework_TestListener
{
    private $firstTest = true;

    public function __construct($out = null)
    {
        parent::__construct($out);
    }

    /**
     * Flush buffer and close output.
     *
     */
    public function flush()
    {
    }

    /**
     * An error occurred.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     */
    public function addError(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        if ($test instanceof PHPUnit_Framework_SelfDescribing) {
            $buffer = $test->toString() . "\n";
        } else {
            $buffer = '';
        }

        $buffer .= "EXCEPTION START\n" . PHPUnit_Framework_TestFailure::exceptionToString($e) .
                   "EXCEPTION END\nSTACK TRACE START\n" .
                   StackTraceFilter::getFilteredStacktrace($e)
                   . "\nSTACK TRACE END\n";

        $this->write("FAIL START\n" . $buffer . "FAIL END\n");
    }

    /**
     * A failure occurred.
     *
     * @param  PHPUnit_Framework_Test                 $test
     * @param  PHPUnit_Framework_AssertionFailedError $e
     * @param  float                                  $time
     */
    public function addFailure(PHPUnit_Framework_Test $test, PHPUnit_Framework_AssertionFailedError $e, $time)
    {
        if (!$test instanceof PHPUnit_Framework_Warning) {
            if ($test instanceof PHPUnit_Framework_SelfDescribing) {
                $buffer = $test->toString() . "\n";
            } else {
                $buffer = '';
            }

            $buffer .= "ASSERTION EXCEPTION START\n" . PHPUnit_Framework_TestFailure::exceptionToString($e) .
                       "ASSERTION EXCEPTION END\nSTACK TRACE START\n" .
                       StackTraceFilter::getFilteredStacktrace($e)
                       . "\nSTACK TRACE END\n";

            $this->write("FAIL START\n" . $buffer . "FAIL END\n");
        }
    }

    /**
     * Incomplete test.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     */
    public function addIncompleteTest(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        $this->write("INCOMPLETE START\n");
        if ($e !== null) {
            $this->write(PHPUnit_Framework_TestFailure::exceptionToString($e));
        }
        $this->write("INCOMPLETE END\n");
    }

    /**
     * Risky test.
     *
     * @param PHPUnit_Framework_Test $test
     * @param Exception              $e
     * @param float                  $time
     * @since  Method available since Release 4.0.0
     */
    public function addRiskyTest(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        $this->write("RISKY START\n");
        if ($e !== null) {
            $this->write(PHPUnit_Framework_TestFailure::exceptionToString($e));
        }
        $this->write("RISKY END\n");
    }

    /**
     * Skipped test.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     * @since  Method available since Release 3.0.0
     */
    public function addSkippedTest(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        $this->write("SKIPPED START\n");
        if ($e !== null) {
            $this->write(PHPUnit_Framework_TestFailure::exceptionToString($e));
        }
        $this->write("SKIPPED END\n");
    }

    /**
     * A testsuite started.
     *
     * @param  PHPUnit_Framework_TestSuite $suite
     * @since  Method available since Release 2.2.0
     */
    public function startTestSuite(PHPUnit_Framework_TestSuite $suite)
    {
    }

    /**
     * A testsuite ended.
     *
     * @param  PHPUnit_Framework_TestSuite $suite
     * @since  Method available since Release 2.2.0
     */
    public function endTestSuite(PHPUnit_Framework_TestSuite $suite)
    {
    }

    /**
     * A test started.
     *
     * @param  PHPUnit_Framework_Test $test
     */
    public function startTest(PHPUnit_Framework_Test $test)
    {
        if ($this->firstTest) {
            $this->write("TESTS START\n");
            $this->firstTest = false;
        }

        //if (!$test instanceof PHPUnit_Framework_Warning) {    // causes endTest() reporting wrong tests
            $name = $test->getName();
            if ($test instanceof PHPUnit_Framework_TestCase) {
                $class = new ReflectionClass($test);
                $name = $class->getName() . "::" . $test->getName(false) . ";" . $class->getFileName();
            }

            $this->write("test:" . $name . "\n");

            if ($class !== null) {
                $method = $class->getMethod($test->getName(false));
                $this->write("line:" . $method->getStartLine() . "\n");
            }
        //}
    }

    /**
     * A test ended.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  float                  $time
     */
    public function endTest(PHPUnit_Framework_Test $test, $time)
    {
        $this->write("time:{$time}\n");
        // $this->write("assertions:{$test->getNumAssertions()}\n");
        $this->write("TEST END\n");
    }
}
